--[[
  Premo Power version 2.0.2 release
  Unit Power movable bar for Paladins and Priests
  By Azmaedus 
  6.0.3 compatible
--]]

-- Lua APIs
local tinsert, tconcat, tremove = table.insert, table.concat, table.remove
local fmt, tostring = string.format, tostring
local select, pairs, next, type, unpack = select, pairs, next, type, unpack
local loadstring, assert, error = loadstring, assert, error
local setmetatable, getmetatable, rawset, rawget = setmetatable, getmetatable, rawset, rawget

-- Load the Ace libraries --
PremoPower = LibStub("AceAddon-3.0"):NewAddon("PremoPower", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("PremoPower")

local LFX = LibStub("LibFrameX-1.0")
if not LFX then return end --fail

local LTC = LibStub("LibTColor-1.0")
if not LTC then return end --fail

-- ---------------------------------------------
-- PremoPower globals
-- ---------------------------------------------
PremoPower.gVersion = GetAddOnMetadata("PremoPower", "Version")
PremoPower.gVersionID = {2,0,0,2,1} -- release major, release minor, build major, build minor, type (alpha 0, beta 1, release 2)
PremoPower.gCurReleaseDate = "01/25/2015"
_G["PremoPower"] = PremoPower

--local currentSpecName = currentSpec and select(2, GetSpecializationInfo(currentSpec)) or "None"
--name, iconTexture, tier, column, rank, maxRank, isExceptional, meetsPrereq, previewRank, meetsPreviewPrereq = GetTalentInfo(tabIndex, talentIndex, inspect, pet, talentGroup)
--GetNumTalentTabs()

local BackDropAlpha = 0.3

local CLASS_PALADIN = "PALADIN"
local CLASS_MONK = "MONK"
local CLASS_ROGUE = "ROGUE"
local CLASS_PRIEST = "PRIEST"

-- Class specific
local MAX_PALADIN_BLIP = 5
local MAX_PRIEST_DISC_BLIP = 5
local MAX_ROGUE_BLIP = 5

local SPEC_PRIEST_DISC = 2
local SPEC_PRIEST_SHADOW = 3

local moduleOptions = {}    	-- Table for [Load On Demand] module options registration

-- ---------------------------------------------
-- create minimap icon
-- ---------------------------------------------
function PremoPower:CreateMinimapIcon()
	local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
	local plugin = ldb:NewDataObject("PremoPower", {
		type = "data source",
		text = "0",
		icon = "Interface\\AddOns\\PremoPower\\media\\icon",
	})

	function plugin.OnClick(self, button)
		if button == "LeftButton" then
			PremoPower:ShowConfig()
		elseif button == "RightButton" then
			if PremoPower.db.profile.frame.hideframes then
				LFX:setVisible(true)
				PremoPower:OnUpdateFrames()
				PremoPower.db.profile.frame.hideframes = false
			else
				LFX:setVisible(false)
				PremoPower.db.profile.frame.hideframes = true
			end
		end
	end

	local hint = L["ICON_CLICK"]
	local hint2 = L["ICON_RCLICK"]
	function plugin.OnTooltipShow(tt)
		tt:AddLine(L["ADDON_NAME"])
		tt:AddLine(" ")
		tt:AddLine(hint, 0.2, 1, 0.2, 1)
		tt:AddLine(" ")
		tt:AddLine(hint2, 0.2, 1, 0.2, 1)
	end

	local f = CreateFrame("Frame")
	f:SetScript("OnEvent", function()
		PremoPower.icon = LibStub("LibDBIcon-1.0", true)
		if not PremoPowerLDBIconDB then PremoPowerLDBIconDB = {} end
		PremoPower.icon:Register("PremoPower", plugin, PremoPowerLDBIconDB)
	end)
	f:RegisterEvent("PLAYER_LOGIN")
end

-- ---------------------------------------------
-- Create the default settings for this addon 
-- ---------------------------------------------
local defaults = { 
    profile =  {
		frame = {
			saved = true,
			pos = {
				x = 0,
				y = 0,
				scale = 1,
				a = "TOPLEFT",
				r = UIParent,
				rp = "CENTER",
				lock = false,
			},
			dim = {
				w = 20,
				h = 20,
				s = 1,
			},
			coloron = {
				r = 255,
				g = 255,
				b = 255,
				a = 1,
			},
			coloroff = {
				r = 255,
				g = 255,
				b = 255,
				a = 1,
			},
			hidetext = false,
		},
		hideframes = false,
		minimap = {
			hide = false,
		},
	},
}

-- LFX tables
local lfxt = {
	{
		root = true, -- creates the frame but doesn't draw any backdrop
		ftype = "Frame",
		name = "PP_BG",
		childbd = {
			bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
			edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]],
			tile = true, tileSize = 4, edgeSize = 4,
			insets = { left = 1, right = 1, top = 1, bottom = 1 },
		},
		rgba = {r = 0, g = 0, b = 0, a = 0.7},
		rgbashaded = {r = 0, g = 0, b = 0, a = 0.7},
		childrgba = {r = 246, g = 227, b = 19, a = 0.7},
		childrgbashaded = {r = 246, g = 227, b = 19, a = 0.3},
		childbrgba = {r = 0, g = 0, b = 0, a = 1},
		childtext = {
			show = true,
			str = "",
			color = {
				r = 255,
				g = 255,
				b = 255,
				a = 1,
			},
			fs = "NumberFont_Shadow_Small",
			halign = "CENTER",
			valign = "CENTER",
		},
		dim = {w = 200, h = 27, s = 1},
		childdim = {w = 25, h = 25, s = 1},
		pos = {x = 0, y = 0, a = "TOPLEFT", r = UIParent, rp = "CENTER"},
		brgba = {r = 255, g = 255, b = 255, a = 0.7},
		show = true,
		movable = {
			enabled = true, 
			drag = {"RightButton"},
		},
		scripts = {
			{
				event = "OnDragStart", 
				func = function( self )
					if not PremoPower.db.profile.frame.pos.lock and IsAltKeyDown() then
						self:StartMoving()
					end
				end,
			},
			{
				event = "OnDragStop", 
				func = function( self )
					if not PremoPower.db.profile.frame.pos.lock then
						self:StopMovingOrSizing()
						PremoPower.db.profile.frame.pos.a, PremoPower.db.profile.frame.pos.r, PremoPower.db.profile.frame.pos.rp, PremoPower.db.profile.frame.pos.x, PremoPower.db.profile.frame.pos.y = self:GetPoint(1)
					end
				end,
			},
		},
	},
	{
		ftype = "Frame",
		name = "PP_Frame1",
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG", parenttext = true,
		pos = {x = 0, y = 1, a = "BOTTOMLEFT", r = "PP_BG", rp = "BOTTOMLEFT"},
		show = true,
	},
	{
		ftype = "Frame",
		name = "PP_Frame2",
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG", parenttext = true,
		pos = {x = 0, y = 0, a = "BOTTOMLEFT", r = "PP_Frame1", rp = "BOTTOMRIGHT"},
		show = true,
	},
	{
		ftype = "Frame",
		name = "PP_Frame3",
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG", parenttext = true,
		pos = {x = 0, y = 0, a = "BOTTOMLEFT", r = "PP_Frame2", rp = "BOTTOMRIGHT"},
		show = true,
	},
	{
		ftype = "Frame",
		name = "PP_Frame4",
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG", parenttext = true,
		pos = {x = 0, y = 0, a = "BOTTOMLEFT", r = "PP_Frame3", rp = "BOTTOMRIGHT"},
		show = true,
	},
	{
		ftype = "Frame",
		name = "PP_Frame5",
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG", parenttext = true,
		pos = {x = 0, y = 0, a = "BOTTOMLEFT", r = "PP_Frame4", rp = "BOTTOMRIGHT"},
		show = true,
	},
	{
		ftype = "Frame",
		name = "PP_Frame6",
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG", parenttext = true,
		pos = {x = 0, y = 0, a = "BOTTOMLEFT", r = "PP_Frame5", rp = "BOTTOMRIGHT"},
		show = true,
	},
}

--[[=============================================================
						OnInitialize
    =============================================================]]
function PremoPower:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("PremoPowerDB", defaults, true)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("PremoPowerDB", self.GenerateOptions)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PremoPower", self.slashcmd, "ppower")
	
	local ACD3 = LibStub("AceConfigDialog-3.0")
	self.optionsFrames = {}
	self.optionsFrames.PremoPower = ACD3:AddToBlizOptions("PremoPowerDB", L["ADDON_NAME"], nil)
	self:RegisterModuleOptions("PPSlashCmd", self.slashcmd, L["ADDON_SLASHCMD"])
	self:RegisterModuleOptions("Profiles", function() return LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db) end, L["Profiles"])
	self.optionsFrames.About = ACD3:AddToBlizOptions("PremoPowerDB", L["OPT_ABOUT"], L["ADDON_NAME"], "About")

	self.gPlayerInfo = {
	  _, _, class = string.upper(UnitClass( 'player' )),
	  level = UnitLevel("player"),
	  spec = GetSpecialization(),
	}

	self.gBackDrop = {
		bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
		edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]],
		tile = true, tileSize = 4, edgeSize = 4,
		insets = { left = 1, right = 1, top = 1, bottom = 1 }
	}

	c = self.gPlayerInfo.class
	if c == CLASS_PALADIN or c == CLASS_MONK or c == CLASS_ROGUE or c == CLASS_PRIEST then
		if self.db.profile.frame.coloron and self.db.profile.frame.coloroff then	
			lfxt[1].childrgba = self.db.profile.frame.coloron
			lfxt[1].childrgbashaded = self.db.profile.frame.coloroff
		elseif self.db.profile.frame.dim then
			lfxt[1].childdim = self.db.profile.frame.dim
		end
		
		LFX:makeFrames(lfxt)
		for i = 1, 6 do
			LFX:setFrameStr(i + 1, tostring(i))
		end
		self:OnUpdateFrames()
		local pos = self.db.profile.frame.pos
		LFX:setPosition(1, pos.a, pos.r, pos.rp, pos.x, pos.y)
	end

	self:Print(L["ADDON_NAME"] .. " " .. self.gVersion .. " " .. L["ADDON_LOADED"])
	
	self:CreateMinimapIcon()
	
	self.OnInitialize = nil
end

function PremoPower:RegisterModuleOptions(name, optionTbl, displayName)
	if moduleOptions then
		moduleOptions[name] = optionTbl
	else
		self.options.args[name] = (type(optionTbl) == "function") and optionTbl() or optionTbl
	end
	self.optionsFrames[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PremoPowerDB", displayName, L["ADDON_NAME"], name)
end

function PremoPower.GenerateOptions()
	if not PremoPower.options then
		PremoPower.GenerateOptionsInternal()
		PremoPower.GenerateOptionsInternal = nil
		moduleOptions = nil
	end
	return PremoPower.options
end

-- Option table for the slash command only
PremoPower.slashcmd = {
	type = "group",
	name = L["ADDON_SLASHCMD"],
	order = -3,
	func = function() PremoPower:ShowConfig() end,
	args = {
		intro = {
			order = 1,
			type = "description",
			name = L["ADDON_SLASHCMD_DESC"],
			cmdHidden = true,
		},
		showicon = {
			order = 6,
			type = "execute",
			name = L["CMD_MINIMAP_SHOW"],
			desc = L["CMD_MINIMAP_SHOW_DESC"],
			func = function()
				PremoPower.db.profile.minimap.hide = false
				PremoPower.icon:Show("PremoPower")
			end,
		},
		hideicon = {
			order = 7,
			type = "execute",
			name = L["CMD_MINIMAP_HIDE"],
			desc = L["CMD_MINIMAP_HIDE_DESC"],
			func = function()
				PremoPower.db.profile.minimap.hide = true 
				PremoPower.icon:Hide("PremoPower")
			end,
		},
		lock = {
			order = 8,
			type = "execute",
			name = L["CMD_LOCK"],
			desc = L["CMD_LOCK_DESC"],
			func = function(...)
				PremoPower.db.profile.minimap.lock = args[1]
			end,
		},
		config = {
			type = "execute",
			name = L["CMD_CONFIG"],
			desc = L["CMD_CONFIG_DESC"],
			func = function() PremoPower:ShowConfig() end,
			guiHidden = true,
		},
	},
}

function PremoPower.GenerateOptionsInternal()
	-- Create the options variable to use in the config screen --
	PremoPower.options = {
		name = L["OPT_PREMOPOWER_OPTIONS"],
		handler = PremoPower,
		type = "group",
		args = {
			PremoPower = {
				name = L["ADDON_NAME"],
				desc = L["OPT_PP_DESC"],
				type = "group",
				order = 1,
				args = {
					hdrcolor = {
						name = L["HDR_COLOR"],
						type = "header",
						order = 1,
					},
					coloron = {
						type = "color",
						name = L["COLOR_ON"],
						hasAlpha = true,
						get = function(info)
							local c = PremoPower.db.profile.frame.coloron
							return c.r/256, c.g/256, c.b/256, c.a
						end,
						set = function(info, r, g, b, a)
							local c = PremoPower.db.profile.frame.coloron
							c.r, c.g, c.b, c.a = r, g, b, a
							PremoPower:setLFXBDColors(1, c, "on")
							PremoPower:OnUpdateFrames()
						end,
						order = 2,
						width = "full",
					},
					coloroff = {
						type = "color",
						name = L["COLOR_OFF"],
						hasAlpha = true,
						get = function(info)
							local c = PremoPower.db.profile.frame.coloroff
							return c.r/256, c.g/256, c.b/256, c.a
						end,
						set = function(info, r, g, b, a)
							local c = PremoPower.db.profile.frame.coloroff
							c.r, c.g, c.b, c.a = r, g, b, a
							PremoPower:setLFXBDColors(1, c, "off")
							PremoPower:OnUpdateFrames()
						end,
						order = 3,
						width = "full",
					},
					hdrdim = {
						name = L["HDR_DIM"],
						type = "header",
						order = 10,
					},
					width = {
						name = L["INP_WIDTH"],
						type = "input",
						width = "half",
						set = function(info, value)	
							dim = PremoPower.db.profile.frame.dim
							dim.w = value 
							PremoPower:setLFXFrameDim(1, dim)
							PremoPower:OnUpdateFrames()
						end,
						get = function(info) return PremoPower.db.profile.frame.dim.w end,
						order = 11,
					},
					height = {
						name = L["INP_HEIGHT"],
						type = "input",
						width = "half",
						set = function(info, value)	
							dim = PremoPower.db.profile.frame.dim
							dim.h = value 
							PremoPower:setLFXFrameDim(1, dim)
							PremoPower:OnUpdateFrames()
						end,
						get = function(info) return PremoPower.db.profile.frame.dim.h end,
						order = 14,
					},
					scale = {
						name = L["INP_SCALE"],
						type = "input",
						width = "half",
						set = function(info, value)
							dim = PremoPower.db.profile.frame.dim
							dim.s = value 
							PremoPower:setLFXFrameDim(1, dim)
						end,
						get = function(info) return PremoPower.db.profile.frame.dim.s end,
						order = 17,
					},
					-- hidetext = {
						-- type = "toggle",
						-- name = L["TOG_HIDETEXT"],
						-- desc = L["TOG_HIDETEXT_DESC"],
						-- set = function(info, value)
							-- PremoPower.db.profile.frame.hidetext = value
							-- PremoPower:OnUpdateFrames()
						-- end,
						-- get = function(info) return PremoPower.db.profile.frame.hidetext end,
						-- width = "normal",
						-- order = 20,
					-- },
				},
			},
			About = {
				name = L["OPT_ABOUT"],
				desc = "",
				type = "group",
				order = 3,
				args = {
					desctitle = {
						name = LTC:setColor( L["ADDON_NAME"] .. " " .. PremoPower.gVersion .. " - " .. PremoPower.gCurReleaseDate, LTC_GREEN ),
						type = "description",
						width = "full",
						fontSize = "medium",
						order = 1,
					},
					descauthor = {
						name = LTC:setColor(L["ADDON_AUTHOR"] .. " " .. "Azmaedus", LTC_BLUE),
						type = "description",
						width = "full",
						fontSize = "medium",
						order = 2,
					},
					descslashhdr = {
						name = "\r" .. L["ABOUT_SLASHCMD_TITLE"],
						type = "description",
						width = "full",
						fontSize = "medium",
						order = 3,
					},
					descslashplay = {
						name = L["ABOUT_SLASHCMD_CONFIG"],
						type = "description",
						width = "full",
						fontSize = "small",
						order = 4,
					},
				},
			},
		},
	}

	for k, v in pairs(moduleOptions) do
		PremoPower.options.args[k] = (type(v) == "function") and v() or v
	end

	-- Add ordering data to the option table generated by AceDBOptions-3.0
	PremoPower.options.args.Profiles.order = -2
end

function PremoPower:ShowConfig()
	-- Open the profiles tab before, so the menu expands
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.Profiles)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.PremoPower)
end

--[[=============================================================
						OnEnable
    =============================================================]]
function PremoPower:OnEnable()
	self:RegisterEvent("UNIT_COMBO_POINTS", "OnUpdateFrames")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "OnUpdateFrames")
	self:RegisterEvent("UNIT_POWER", "OnUpdateFrames")
	self:RegisterEvent("UNIT_AURA", "OnUpdateFrames")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "OnUpdateFrames")
	
	self:OnUpdateFrames()
end

--[[=============================================================
						OnDisable
    =============================================================]]
function PremoPower:OnDisable()
	self:UnregisterEvent("UNIT_COMBO_POINTS")
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	self:UnregisterEvent("UNIT_POWER")
	self:UnregisterEvent("UNIT_AURA")
	self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
end

-- -------------------------------------------
-- Events
-- -------------------------------------------
function PremoPower:OnProfileChanged(event, database, newProfileKey)
	db = database.profile
	self.db.profile.frame = db.frame

	c = self.gPlayerInfo.class
	if c == CLASS_PALADIN or c == CLASS_MONK or c == CLASS_ROGUE or c == CLASS_PRIEST then
		if self.db.profile.frame.coloron and self.db.profile.frame.coloroff then	
			lfxt[1].childrgba = self.db.profile.frame.coloron
			lfxt[1].childrgbashaded = self.db.profile.frame.coloroff
		elseif self.db.profile.frame.dim then
			lfxt[1].childdim = self.db.profile.frame.dim
		end

		LFX:makeFrames(lfxt)
		for i = 1, 6 do
			LFX:setFrameStr(i + 1, tostring(i))
		end
		local pos = self.db.profile.frame.pos
		LFX:setPosition(1, pos.a, pos.r, pos.rp, pos.x, pos.y)
	end
end
	
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

function PremoPower:OnUpdateFrames()
	c = self.gPlayerInfo.class 
	s = PremoPower.gPlayerInfo.spec
	if c == CLASS_ROGUE then
		self:updateFrames( MAX_ROGUE_BLIP, GetComboPoints("player" ,"target"), 1 ) -- number of frames to remove from calculation
	elseif c == CLASS_MONK then
		self:updateFrames( UnitPowerMax("player", SPELL_POWER_CHI), UnitPower("player", SPELL_POWER_CHI), 1 ) -- number of frames to remove from calculation
	elseif c == CLASS_PALADIN then
		self:updateFrames( UnitPowerMax("player", SPELL_POWER_HOLY_POWER), UnitPower("player", SPELL_POWER_HOLY_POWER), 1 ) -- number of frames to remove from calculation
	elseif c == CLASS_PRIEST then
		if s == SPEC_PRIEST_DISC then -- disc
			self:updateFrames( UnitPowerMax("player", SPELL_POWER_SHADOW_ORBS), UnitPower( "player", SPELL_POWER_SHADOW_ORBS ), 1 ) -- number of frames to remove from calculation
		elseif s == SPEC_PRIEST_SHADOW then -- shadow
			self:updateFrames( MAX_PRIEST_DISC_BLIP, getEvangelismStacks(), 1 ) -- number of frames to remove from calculation
		end
	end
end

function PremoPower:getEvangelismStacks()
	local _, _, _, count = UnitAura( "player", "Evangelism" ) --81661
	if count == nil then
		return 0
	else
		return count
	end
end

-- Function to update frames in regards to # of frames/state required
-- total: maximum frames to display
-- [cur]: shadow states for frames after cur(index)
-- [minus]: used as a balance if we do not want to calculate cur with a root
function PremoPower:updateFrames(total, cur, minus)
	total = total + minus
	cur = cur + minus
	for i = 1, #LFX.tframes do
		v = LFX.tframes[i]
		if not v.root then
			local rgba = {}
			local rgbashaded = {}
			local dim = {}
			
			-- This is somewhat ugly, I can do better
			-- parent info or not (3x findFrame)
			if v.parentrgba then
				local id = LFX:findFrame(v.parentname)
				if id then
					rgba = LFX.tframes[id].childrgba
					rgbashaded = LFX.tframes[id].childrgbashaded
				else
					rgba = v.rgba
					rgbashaded = v.rgbashaded
				end
			else
				rgba = v.rgba
				rgbashaded = v.rgbashaded
			end

			if v.parentdim then
				local id = LFX:findFrame(v.parentname)
				if id then
					dim = LFX.tframes[id].childdim
				else
					dim = v.dim
				end
			else
				dim = v.dim
			end

			-- set backdrop color and dimesions
			if i <= cur then
				v:Show()
				v:SetBackdropColor(rgba.r/256, rgba.g/256, rgba.b/256, rgba.a)
			elseif i <= total then
				v:Show()
				v:SetBackdropColor(rgbashaded.r/256, rgbashaded.g/256, rgbashaded.b/256, rgbashaded.a)
			elseif i > total then
				v:Hide()
			end
			
			if i <= total then
				v:SetWidth(dim.w)
				v:SetHeight(dim.h)
				v:SetScale(dim.s)
			end
		end
	end
end

-- c = "on" or "off"
function PremoPower:setLFXBDColors(i, c, t)
	local lfx = {}
	if t == "on" then
		lfx = LFX.tframes[i].childrgba
	elseif t == "off" then
		lfx = LFX.tframes[i].childrgbashaded
	end
	lfx.r = c.r*255
	lfx.g = c.g*255
	lfx.b = c.b*255
	lfx.a = c.a
end

function PremoPower:setLFXTextColor(i, c, t)
	local lfx = {}
	if t == "on" then
		lfx = LFX.tframes[i].childtrgba
	end
	lfx.r = c.r*255
	lfx.g = c.g*255
	lfx.b = c.b*255
	lfx.a = c.a
end

function PremoPower:setLFXFrameDim(i, d)
	local lfx = {}
	lfx = LFX.tframes[i].childdim
	lfx.w = d.w
	lfx.h = d.h
	lfx.s = d.s
end